@@ -1,7 +1,7 @@
 /* loopback.c - command to add loopback devices.  */
 /*
  *  GRUB  --  GRand Unified Bootloader
- *  Copyright (C) 2005,2006  Free Software Foundation, Inc.
+ *  Copyright (C) 2005  Free Software Foundation, Inc.
  *
  *  GRUB is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -53,12 +53,10 @@ delete_loopback (const char *name)
 
   /* Search for the device.  */
   for (dev = loopback_list, prev = &loopback_list;
-       dev;
-       prev = &dev->next, dev = dev->next)
-    if (grub_strcmp (dev->devname, name) == 0)
+       dev; prev = &dev->next, dev = dev->next)
+    if (!grub_strcmp (dev->devname, name))
       break;
-  
-  if (! dev)
+  if (!dev)
     return grub_error (GRUB_ERR_BAD_DEVICE, "Device not found");
   
   /* Remove the device from the list.  */
@@ -98,13 +96,12 @@ grub_cmd_loopback (struct grub_arg_list *state,
   
   /* First try to replace the old device.  */
   for (newdev = loopback_list; newdev; newdev = newdev->next)
-    if (grub_strcmp (newdev->devname, args[0]) == 0)
+    if (!grub_strcmp (newdev->devname, args[0]))
       break;
-  
   if (newdev)
     {
       char *newname = grub_strdup (args[1]);
-      if (! newname)
+      if (!newname)
 	return grub_errno;
       
       grub_free (newdev->filename);
@@ -118,18 +115,18 @@ grub_cmd_loopback (struct grub_arg_list *state,
   
   /* Unable to replace it, make a new entry.  */
   newdev = grub_malloc (sizeof (struct grub_loopback));
-  if (! newdev)
+  if (!newdev)
     return grub_errno;
   
   newdev->devname = grub_strdup (args[0]);
-  if (! newdev->devname)
+  if (!newdev->devname)
     {
       grub_free (newdev);
       return grub_errno;
     }
   
   newdev->filename = grub_strdup (args[1]);
-  if (! newdev->filename)
+  if (!newdev->devname)
     {
       grub_free (newdev->devname);
       grub_free (newdev);
@@ -166,7 +163,7 @@ grub_loopback_open (const char *name, grub_disk_t disk)
   struct grub_loopback *dev;
     
   for (dev = loopback_list; dev; dev = dev->next)
-    if (grub_strcmp (dev->devname, name) == 0)
+    if (!grub_strcmp (dev->devname, name))
       break;
   
   if (! dev)
@@ -196,26 +193,26 @@ grub_loopback_close (grub_disk_t disk)
 }
 
 static grub_err_t
-grub_loopback_read (grub_disk_t disk, grub_disk_addr_t sector,
-		    grub_size_t size, char *buf)
+grub_loopback_read (grub_disk_t disk, unsigned long sector,
+		    unsigned long size, char *buf)
 {
   grub_file_t file = (grub_file_t) disk->data;
-  grub_off_t pos;
+  long pos;
   
-  grub_file_seek (file, sector << GRUB_DISK_SECTOR_BITS);
+  grub_file_seek (file, sector * GRUB_DISK_SECTOR_SIZE);
   
-  grub_file_read (file, buf, size << GRUB_DISK_SECTOR_BITS);
+  grub_file_read (file, buf, size * GRUB_DISK_SECTOR_SIZE);
   if (grub_errno)
     return grub_errno;
   
   /* In case there is more data read than there is available, in case
      of files that are not a multiple of GRUB_DISK_SECTOR_SIZE, fill
      the rest with zeros.  */
-  pos = (sector + size) << GRUB_DISK_SECTOR_BITS;
+  pos = sector * GRUB_DISK_SECTOR_SIZE + size * GRUB_DISK_SECTOR_SIZE;
   if (pos > file->size)
     {
-      grub_size_t amount = pos - file->size;
-      grub_memset (buf + size - amount, 0, amount);
+      unsigned long amount = pos - file->size;
+      grub_memset (buf + pos - amount, amount, 0);
     }
   
   return 0;
@@ -223,9 +220,9 @@ grub_loopback_read (grub_disk_t disk, grub_disk_addr_t sector,
 
 static grub_err_t
 grub_loopback_write (grub_disk_t disk __attribute ((unused)),
-		     grub_disk_addr_t sector __attribute ((unused)),
-		     grub_size_t size __attribute ((unused)),
-		     const char *buf __attribute ((unused)))
+		   unsigned long sector __attribute ((unused)),
+		   unsigned long size __attribute ((unused)),
+		   const char *buf __attribute ((unused)))
 {
   return GRUB_ERR_NOT_IMPLEMENTED_YET;
 }
@@ -246,7 +243,7 @@ static struct grub_disk_dev grub_loopback_dev =
 
 GRUB_MOD_INIT(loop)
 {
-  (void) mod;			/* To stop warning. */
+  (void)mod;			/* To stop warning. */
   grub_register_command ("loopback", grub_cmd_loopback, GRUB_COMMAND_FLAG_BOTH,
 			 "loopback [-d|-p] DEVICENAME FILE",
 			 "Make a device of a file.", options);
@@ -258,3 +255,4 @@ GRUB_MOD_FINI(loop)
   grub_unregister_command ("loopback");
   grub_disk_dev_unregister (&grub_loopback_dev);
 }
+
