@@ -1,20 +1,21 @@
 /* affs.c - Amiga Fast FileSystem.  */
 /*
  *  GRUB  --  GRand Unified Bootloader
- *  Copyright (C) 2005,2006,2007  Free Software Foundation, Inc.
+ *  Copyright (C) 2005  Free Software Foundation, Inc.
  *
- *  GRUB is free software: you can redistribute it and/or modify
+ *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation, either version 3 of the License, or
+ *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
  *
- *  GRUB is distributed in the hope that it will be useful,
+ *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with GRUB.  If not, see <http://www.gnu.org/licenses/>.
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
 #include <grub/err.h>
@@ -149,9 +150,9 @@ grub_affs_read_block (grub_fshelp_node_t node, int fileblock)
    POS.  Return the amount of read bytes in READ.  */
 static grub_ssize_t
 grub_affs_read_file (grub_fshelp_node_t node,
-		     void NESTED_FUNC_ATTR (*read_hook) (grub_disk_addr_t sector,
+		     void (*read_hook) (unsigned long sector,
 					unsigned offset, unsigned length),
-		     int pos, grub_size_t len, char *buf)
+		     int pos, unsigned int len, char *buf)
 {
   return grub_fshelp_read_file (node->data->disk, node, read_hook,
 				pos, len, buf, grub_affs_read_block,
@@ -248,9 +249,6 @@ grub_affs_mount (grub_disk_t disk)
   return data;
 
  fail:
-  if (grub_errno == GRUB_ERR_OUT_OF_RANGE)
-    grub_error (GRUB_ERR_BAD_FS, "not an affs filesystem");
-
   grub_free (data);
   grub_free (rootblock);
   return 0;
@@ -441,7 +439,7 @@ grub_affs_close (grub_file_t file)
 
 /* Read LEN bytes data from FILE into BUF.  */
 static grub_ssize_t
-grub_affs_read (grub_file_t file, char *buf, grub_size_t len)
+grub_affs_read (grub_file_t file, char *buf, grub_ssize_t len)
 {
   struct grub_affs_data *data = 
     (struct grub_affs_data *) file->data;
