@@ -1,7 +1,7 @@
 /* hfs.c - HFS.  */
 /*
  *  GRUB  --  GRand Unified Bootloader
- *  Copyright (C) 2004, 2005  Free Software Foundation, Inc.
+ *  Copyright (C) 2004, 2005, 2006  Free Software Foundation, Inc.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -18,6 +18,9 @@
  *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
+/* HFS is documented at
+   http://developer.apple.com/documentation/mac/Files/Files-2.html */
+
 #include <grub/err.h>
 #include <grub/file.h>
 #include <grub/mm.h>
@@ -25,9 +28,10 @@
 #include <grub/disk.h>
 #include <grub/dl.h>
 #include <grub/types.h>
+#include <grub/hfs.h>
 
 #define	GRUB_HFS_SBLOCK		2
-#define GRUB_HFS_MAGIC		0x4244
+#define GRUB_HFS_EMBED_HFSPLUS_SIG 0x482B
 
 #define GRUB_HFS_BLKS		(data->blksz >> 9)
 
@@ -40,37 +44,6 @@ enum
     GRUB_HFS_FILETYPE_FILE = 2
   };
 
-/* A single extent.  A file consists of suchs extents.  */
-struct grub_hfs_extent
-{
-  /* The first physical block.  */
-  grub_uint16_t first_block;
-  grub_uint16_t count;
-};
-
-/* HFS stores extents in groups of 3.  */
-typedef struct grub_hfs_extent grub_hfs_datarecord_t[3];
-
-/* The HFS superblock (The official name is `Master Directory
-   Block').  */
-struct grub_hfs_sblock
-{
-  grub_uint16_t magic;
-  grub_uint8_t unused[18];
-  grub_uint32_t blksz;
-  grub_uint8_t unused2[4];
-  grub_uint16_t first_block;
-  grub_uint8_t unused4[6];
-
-  /* A pascal style string that holds the volumename.  */
-  grub_uint8_t volname[28];
-  
-  grub_uint8_t unused5[70];
-  grub_hfs_datarecord_t extent_recs;
-  grub_uint32_t catalog_size;
-  grub_hfs_datarecord_t catalog_recs;
-} __attribute__ ((packed));
-
 /* A node desciptor.  This is the header of every node.  */
 struct grub_hfs_node
 {
@@ -345,7 +318,14 @@ grub_hfs_mount (grub_disk_t disk)
   /* Check if this is a HFS filesystem.  */
   if (grub_be_to_cpu16 (data->sblock.magic) != GRUB_HFS_MAGIC)
     {
-      grub_error (GRUB_ERR_BAD_FS, "not a hfs filesystem");
+      grub_error (GRUB_ERR_BAD_FS, "not an HFS filesystem");
+      goto fail;
+    }
+
+  /* Check if this is an embedded HFS+ filesystem.  */
+  if (grub_be_to_cpu16 (data->sblock.embed_sig) == GRUB_HFS_EMBED_HFSPLUS_SIG)
+    {
+      grub_error (GRUB_ERR_BAD_FS, "embedded HFS+ filesystem");
       goto fail;
     }
   
@@ -377,7 +357,7 @@ grub_hfs_mount (grub_disk_t disk)
      volume name.  */
   key.parent_dir = grub_cpu_to_be32 (1);
   key.strlen = data->sblock.volname[0];
-  grub_strcpy (key.str, data->sblock.volname + 1);
+  grub_strcpy ((char *) key.str, (char *) (data->sblock.volname + 1));
   
   if (grub_hfs_find_node (data, (char *) &key, data->cat_root,
 			  0, (char *) &dir, sizeof (dir)) == 0)
@@ -413,7 +393,7 @@ grub_hfs_cmp_catkeys (struct grub_hfs_catalog_key *k1,
   if (cmp != 0)
     return cmp;
   
-  cmp = grub_strncasecmp (k1->str, k2->str, k1->strlen);
+  cmp = grub_strncasecmp ((char *) (k1->str), (char *) (k2->str), k1->strlen);
   
   /* This is required because the compared strings are not of equal
      length.  */
@@ -693,11 +673,11 @@ grub_hfs_find_dir (struct grub_hfs_data *data, const char *path,
       
       key.parent_dir = grub_cpu_to_be32 (inode);
       key.strlen = grub_strlen (path);
-      grub_strcpy (key.str, path);
+      grub_strcpy ((char *) (key.str), path);
       
       /* Lookup this node.  */
-      if (!grub_hfs_find_node (data, (char *) &key, data->cat_root,
-			       0, (char *) &frec, sizeof (frec)))
+      if (! grub_hfs_find_node (data, (char *) &key, data->cat_root,
+				0, (char *) &frec, sizeof (frec)))
 	{
 	  grub_error (GRUB_ERR_FILE_NOT_FOUND, "file not found");
 	  goto fail;
@@ -737,7 +717,7 @@ grub_hfs_dir (grub_device_t device, const char *path,
       char *filetype = rec->data;
       struct grub_hfs_catalog_key *ckey = rec->key;
       
-      grub_strncpy (fname, ckey->str, ckey->strlen);
+      grub_strncpy (fname, (char *) (ckey->str), ckey->strlen);
       
       if (*filetype == GRUB_HFS_FILETYPE_DIR)
 	return hook (fname, 1);
@@ -854,7 +834,8 @@ grub_hfs_label (grub_device_t device, char **label)
   data = grub_hfs_mount (device->disk);
   
   if (data)
-    *label = grub_strndup (data->sblock.volname + 1, *data->sblock.volname);
+    *label = grub_strndup ((char *) (data->sblock.volname + 1),
+			   *data->sblock.volname);
   else
     *label = 0;
 
