@@ -1,20 +1,21 @@
 /* hfsplus.c - HFS+ Filesystem.  */
 /*
  *  GRUB  --  GRand Unified Bootloader
- *  Copyright (C) 2005,2006,2007,2008,2009  Free Software Foundation, Inc.
+ *  Copyright (C) 2005, 2006  Free Software Foundation, Inc.
  *
- *  GRUB is free software: you can redistribute it and/or modify
+ *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation, either version 3 of the License, or
+ *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
  *
- *  GRUB is distributed in the hope that it will be useful,
+ *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with GRUB.  If not, see <http://www.gnu.org/licenses/>.
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
 /* HFS+ is documented at http://developer.apple.com/technotes/tn/tn1150.html */
@@ -57,12 +58,9 @@ struct grub_hfsplus_volheader
   grub_uint16_t magic;
   grub_uint16_t version;
   grub_uint32_t attributes;
-  grub_uint8_t unused1[12];
-  grub_uint32_t utime;
-  grub_uint8_t unused2[16];
+  grub_uint8_t unused[32];
   grub_uint32_t blksize;
-  grub_uint8_t unused3[60];
-  grub_uint64_t num_serial;
+  grub_uint8_t unused2[68];
   struct grub_hfsplus_forkdata allocations_file;
   struct grub_hfsplus_forkdata extents_file;
   struct grub_hfsplus_forkdata catalog_file;
@@ -99,13 +97,6 @@ struct grub_hfsplus_btheader
   grub_uint32_t last_leaf_node;
   grub_uint16_t nodesize;
   grub_uint16_t keysize;
-  grub_uint32_t total_nodes;
-  grub_uint32_t free_nodes;
-  grub_uint16_t reserved1;
-  grub_uint32_t clump_size;  /* ignored */
-  grub_uint8_t btree_type;
-  grub_uint8_t key_compare;
-  grub_uint32_t attributes;
 } __attribute__ ((packed));
 
 /* The on disk layout of a catalog key.  */
@@ -143,11 +134,9 @@ struct grub_hfsplus_catfile
   grub_uint16_t flags;
   grub_uint32_t reserved;
   grub_uint32_t fileid;
-  grub_uint8_t unused1[4];
-  grub_uint32_t mtime;
-  grub_uint8_t unused2[22];
+  grub_uint8_t unused1[30];
   grub_uint16_t mode;
-  grub_uint8_t unused3[44];
+  grub_uint8_t unused2[44];
   struct grub_hfsplus_forkdata data;
   struct grub_hfsplus_forkdata resource;
 } __attribute__ ((packed));
@@ -171,9 +160,6 @@ enum grub_hfsplus_filetype
     GRUB_HFSPLUS_FILETYPE_REG_THREAD = 4
   };
 
-#define GRUB_HFSPLUSX_BINARYCOMPARE	0xBC
-#define GRUB_HFSPLUSX_CASEFOLDING	0xCF
-
 /* Internal representation of a catalog key.  */
 struct grub_hfsplus_catkey_internal
 {
@@ -205,7 +191,6 @@ struct grub_fshelp_node
   struct grub_hfsplus_extent extents[8];
   grub_uint64_t size;
   grub_uint32_t fileid;
-  grub_int32_t mtime;
 };
 
 struct grub_hfsplus_btree
@@ -234,10 +219,11 @@ struct grub_hfsplus_data
   /* This is the offset into the physical disk for an embedded HFS+
      filesystem (one inside a plain HFS wrapper).  */
   int embedded_offset;
-  int case_sensitive;
 };
 
+#ifndef GRUB_UTIL
 static grub_dl_t my_mod;
+#endif
 
 
 /* Return the offset of the record with the index INDEX, in the node
@@ -300,8 +286,8 @@ static int grub_hfsplus_cmp_extkey (struct grub_hfsplus_key *keya,
 
 /* Search for the block FILEBLOCK inside the file NODE.  Return the
    blocknumber of this block on disk.  */
-static grub_disk_addr_t
-grub_hfsplus_read_block (grub_fshelp_node_t node, grub_disk_addr_t fileblock)
+static int
+grub_hfsplus_read_block (grub_fshelp_node_t node, int fileblock)
 {
   struct grub_hfsplus_btnode *nnode = 0;
   int blksleft = fileblock;
@@ -318,10 +304,10 @@ grub_hfsplus_read_block (grub_fshelp_node_t node, grub_disk_addr_t fileblock)
       blk = grub_hfsplus_find_block (extents, &blksleft);
 
       /* The previous iteration of this loop allocated memory.  The
-	 code above used this memory, it can be freed now.  */
+	 code above used this memory, it can be free'ed now.  */
       grub_free (nnode);
       nnode = 0;
-
+      
       if (blk != -1)
 	return (blk
 		+ (node->data->embedded_offset >> (node->data->log2blksize
@@ -371,9 +357,9 @@ grub_hfsplus_read_block (grub_fshelp_node_t node, grub_disk_addr_t fileblock)
    POS.  Return the amount of read bytes in READ.  */
 static grub_ssize_t
 grub_hfsplus_read_file (grub_fshelp_node_t node,
-			void NESTED_FUNC_ATTR (*read_hook) (grub_disk_addr_t sector,
+			void (*read_hook) (unsigned long sector,
 					   unsigned offset, unsigned length),
-			int pos, grub_size_t len, char *buf)
+			int pos, unsigned int len, char *buf)
 {
   return grub_fshelp_read_file (node->data->disk, node, read_hook,
 				pos, len, buf, grub_hfsplus_read_block,
@@ -387,7 +373,6 @@ grub_hfsplus_mount (grub_disk_t disk)
   struct grub_hfsplus_data *data;
   struct grub_hfsplus_btheader header;
   struct grub_hfsplus_btnode node;
-  grub_uint16_t magic;
   union {
     struct grub_hfs_sblock hfs;
     struct grub_hfsplus_volheader hfsplus;
@@ -401,7 +386,7 @@ grub_hfsplus_mount (grub_disk_t disk)
 
   /* Read the bootblock.  */
   grub_disk_read (disk, GRUB_HFSPLUS_SBLOCK, 0, sizeof (volheader),
-		  &volheader);
+		  (char *) &volheader);
   if (grub_errno)
     goto fail;
 
@@ -428,15 +413,15 @@ grub_hfsplus_mount (grub_disk_t disk)
 			       * (ablk_size >> GRUB_DISK_SECTOR_BITS));
 
       grub_disk_read (disk, data->embedded_offset + GRUB_HFSPLUS_SBLOCK, 0,
-		      sizeof (volheader), &volheader);
+		      sizeof (volheader), (char *) &volheader);
       if (grub_errno)
 	goto fail;
     }
 
   /* Make sure this is an HFS+ filesystem.  XXX: Do we really support
      HFX?  */
-  magic = grub_be_to_cpu16 (volheader.hfsplus.magic);
-  if ((magic != GRUB_HFSPLUS_MAGIC) && (magic != GRUB_HFSPLUSX_MAGIC))
+  if ((grub_be_to_cpu16 (volheader.hfsplus.magic) != GRUB_HFSPLUS_MAGIC)
+      && (grub_be_to_cpu16 (volheader.hfsplus.magic) != GRUB_HFSPLUSX_MAGIC))
     {
       grub_error (GRUB_ERR_BAD_FS, "not a HFS+ filesystem");
       goto fail;
@@ -455,7 +440,7 @@ grub_hfsplus_mount (grub_disk_t disk)
   grub_memcpy (&data->catalog_tree.file.extents,
 	       data->volheader.catalog_file.extents,
 	       sizeof data->volheader.catalog_file.extents);
-  data->catalog_tree.file.size =
+  data->catalog_tree.file.size = 
     grub_be_to_cpu64 (data->volheader.catalog_file.size);
 
   /* Make a new node for the extent overflow file.  */
@@ -465,29 +450,27 @@ grub_hfsplus_mount (grub_disk_t disk)
 	       data->volheader.extents_file.extents,
 	       sizeof data->volheader.catalog_file.extents);
 
-  data->extoverflow_tree.file.size =
+  data->extoverflow_tree.file.size = 
     grub_be_to_cpu64 (data->volheader.extents_file.size);
 
   /* Read the essential information about the trees.  */
-  if (grub_hfsplus_read_file (&data->catalog_tree.file, 0,
-			      sizeof (struct grub_hfsplus_btnode),
-			      sizeof (header), (char *) &header) <= 0)
+  if (! grub_hfsplus_read_file (&data->catalog_tree.file, 0,
+				sizeof (struct grub_hfsplus_btnode),
+				sizeof (header), (char *) &header))
     goto fail;
 
   data->catalog_tree.root = grub_be_to_cpu32 (header.root);
   data->catalog_tree.nodesize = grub_be_to_cpu16 (header.nodesize);
-  data->case_sensitive = ((magic == GRUB_HFSPLUSX_MAGIC) &&
-			  (header.key_compare == GRUB_HFSPLUSX_BINARYCOMPARE));
 
-  if (grub_hfsplus_read_file (&data->extoverflow_tree.file, 0,
-			      sizeof (struct grub_hfsplus_btnode),
-			      sizeof (header), (char *) &header) <= 0)
+  if (! grub_hfsplus_read_file (&data->extoverflow_tree.file, 0,
+				sizeof (struct grub_hfsplus_btnode),
+				sizeof (header), (char *) &header))
     goto fail;
 
   data->extoverflow_tree.root = grub_be_to_cpu32 (header.root);
 
-  if (grub_hfsplus_read_file (&data->extoverflow_tree.file, 0, 0,
-			      sizeof (node), (char *) &node) <= 0)
+  if (! grub_hfsplus_read_file (&data->extoverflow_tree.file, 0, 0,
+				sizeof (node), (char *) &node))
     goto fail;
 
   data->extoverflow_tree.root = grub_be_to_cpu32 (header.root);
@@ -499,10 +482,6 @@ grub_hfsplus_mount (grub_disk_t disk)
   return data;
 
  fail:
-
-  if (grub_errno == GRUB_ERR_OUT_OF_RANGE)
-    grub_error (GRUB_ERR_BAD_FS, "not a hfsplus filesystem");
-
   grub_free (data);
   return 0;
 }
@@ -518,12 +497,12 @@ grub_hfsplus_cmp_catkey (struct grub_hfsplus_key *keya,
   char *filename;
   int i;
   int diff;
-
+  
   diff = grub_be_to_cpu32 (catkey_a->parent) - catkey_b->parent;
   if (diff)
-    return diff;
+    return diff; 
 
-  /* Change the filename in keya so the endianness is correct.  */
+  /* Change the filename in keya so the endianess is correct.  */
   for (i = 0; i < grub_be_to_cpu16 (catkey_a->namelen); i++)
     catkey_a->name[i] = grub_be_to_cpu16 (catkey_a->name[i]);
 
@@ -539,7 +518,7 @@ grub_hfsplus_cmp_catkey (struct grub_hfsplus_key *keya,
 
   grub_free (filename);
 
-  /* The endianness was changed to host format, change it back to
+  /* The endianess was changed to host format, change it back to
      whatever it was.  */
   for (i = 0; i < grub_be_to_cpu16 (catkey_a->namelen); i++)
     catkey_a->name[i] = grub_cpu_to_be16 (catkey_a->name[i]);
@@ -592,7 +571,7 @@ grub_hfsplus_btree_iterate_node (struct grub_hfsplus_btree *btree,
 				 int first_rec,
 				 int (*hook) (void *record))
 {
-  int rec;
+  int rec;  
 
   for (;;)
     {
@@ -608,10 +587,10 @@ grub_hfsplus_btree_iterate_node (struct grub_hfsplus_btree *btree,
       if (! first_node->next)
 	break;
 
-      if (grub_hfsplus_read_file (&btree->file, 0,
-				  (grub_be_to_cpu32 (first_node->next)
-				   * btree->nodesize),
-				  btree->nodesize, cnode) <= 0)
+      if (! grub_hfsplus_read_file (&btree->file, 0,
+				    (grub_be_to_cpu32 (first_node->next)
+				     * btree->nodesize),
+				    btree->nodesize, cnode))
 	return 1;
 
       /* Don't skip any record in the next iteration.  */
@@ -624,7 +603,7 @@ grub_hfsplus_btree_iterate_node (struct grub_hfsplus_btree *btree,
 /* Lookup the node described by KEY in the B+ Tree BTREE.  Compare
    keys using the function COMPARE_KEYS.  When a match is found,
    return the node in MATCHNODE and a pointer to the data in this node
-   in KEYOFFSET.  MATCHNODE should be freed by the caller.  */
+   in KEYOFFSET.  MATCHNODE should be free'ed by the caller.  */
 static grub_err_t
 grub_hfsplus_btree_search (struct grub_hfsplus_btree *btree,
 			   struct grub_hfsplus_key_internal *key,
@@ -647,12 +626,12 @@ grub_hfsplus_btree_search (struct grub_hfsplus_btree *btree,
       int match = 0;
 
       /* Read a node.  */
-      if (grub_hfsplus_read_file (&btree->file, 0,
-				  (long)currnode * (long)btree->nodesize,
-				  btree->nodesize, (char *) node) <= 0)
+      if (! grub_hfsplus_read_file (&btree->file, 0,
+				    (long)currnode * (long)btree->nodesize,
+				    btree->nodesize, (char *) node))
 	{
 	  grub_free (node);
-	  return grub_error (GRUB_ERR_BAD_FS, "Couldn't read i-node.");
+	  return grub_errno;
 	}
 
       nodedesc = (struct grub_hfsplus_btnode *) node;
@@ -661,8 +640,8 @@ grub_hfsplus_btree_search (struct grub_hfsplus_btree *btree,
       for (rec = 0; rec < grub_be_to_cpu16 (nodedesc->count); rec++)
 	{
 	  struct grub_hfsplus_key *currkey;
-	  currkey = grub_hfsplus_btree_recptr (btree, nodedesc, rec);
-
+	  currkey = grub_hfsplus_btree_recptr (btree, nodedesc, rec);	
+  
 	  /* The action that has to be taken depend on the type of
 	     record.  */
 	  if (nodedesc->type == GRUB_HFSPLUS_BTNODE_TYPE_LEAF
@@ -679,7 +658,7 @@ grub_hfsplus_btree_search (struct grub_hfsplus_btree *btree,
 	    {
 	      grub_uint32_t *pointer;
 
-	      /* The place where the key could have been found didn't
+	      /* The place where the key could've been found didn't
 		 contain the key.  This means that the previous match
 		 is the one that should be followed.  */
 	      if (compare_keys (currkey, key) > 0)
@@ -716,7 +695,7 @@ grub_hfsplus_iterate_dir (grub_fshelp_node_t dir,
 				   grub_fshelp_node_t node))
 {
   int ret = 0;
-
+  
   auto int list_nodes (void *record);
   int list_nodes (void *record)
     {
@@ -730,7 +709,7 @@ grub_hfsplus_iterate_dir (grub_fshelp_node_t dir,
       catkey = (struct grub_hfsplus_catkey *) record;
 
       fileinfo =
-	(struct grub_hfsplus_catfile *) ((char *) record
+	(struct grub_hfsplus_catfile *) ((char *) record 
 					 + grub_be_to_cpu16 (catkey->keylen)
 					 + 2 + (grub_be_to_cpu16(catkey->keylen)
 						% 2));
@@ -785,10 +764,6 @@ grub_hfsplus_iterate_dir (grub_fshelp_node_t dir,
       for (i = 0; i < grub_be_to_cpu16 (catkey->namelen); i++)
 	catkey->name[i] = grub_be_to_cpu16 (catkey->name[i]);
 
-      /* hfs+ is case insensitive.  */
-      if (! dir->data->case_sensitive)
-	type |= GRUB_FSHELP_CASE_INSENSITIVE;
-
       /* Only accept valid nodes.  */
       if (grub_strlen (filename) == grub_be_to_cpu16 (catkey->namelen))
 	{
@@ -796,10 +771,9 @@ grub_hfsplus_iterate_dir (grub_fshelp_node_t dir,
 	     callback function.  */
 	  node = grub_malloc (sizeof (*node));
 	  node->data = dir->data;
-
+	  
 	  grub_memcpy (node->extents, fileinfo->data.extents,
 		       sizeof (node->extents));
-	  node->mtime = grub_be_to_cpu32 (fileinfo->mtime) - 2082844800;
 	  node->size = grub_be_to_cpu64 (fileinfo->data.size);
 	  node->fileid = grub_be_to_cpu32 (fileinfo->fileid);
 
@@ -839,9 +813,11 @@ grub_hfsplus_open (struct grub_file *file, const char *name)
 {
   struct grub_hfsplus_data *data;
   struct grub_fshelp_node *fdiro = 0;
-
+  
+#ifndef GRUB_UTIL
   grub_dl_ref (my_mod);
-
+#endif
+  
   data = grub_hfsplus_mount (file->device->disk);
   if (!data)
     goto fail;
@@ -865,8 +841,10 @@ grub_hfsplus_open (struct grub_file *file, const char *name)
   if (data && fdiro != &data->dirroot)
     grub_free (fdiro);
   grub_free (data);
-
+  
+#ifndef GRUB_UTIL
   grub_dl_unref (my_mod);
+#endif
 
   return grub_errno;
 }
@@ -877,7 +855,9 @@ grub_hfsplus_close (grub_file_t file)
 {
   grub_free (file->data);
 
+#ifndef GRUB_UTIL
   grub_dl_unref (my_mod);
+#endif
 
   return GRUB_ERR_NONE;
 }
@@ -885,9 +865,9 @@ grub_hfsplus_close (grub_file_t file)
 
 /* Read LEN bytes data from FILE into BUF.  */
 static grub_ssize_t
-grub_hfsplus_read (grub_file_t file, char *buf, grub_size_t len)
+grub_hfsplus_read (grub_file_t file, char *buf, grub_ssize_t len)
 {
-  struct grub_hfsplus_data *data =
+  struct grub_hfsplus_data *data = 
     (struct grub_hfsplus_data *) file->data;
 
   int size = grub_hfsplus_read_file (&data->opened_file, file->read_hook,
@@ -898,13 +878,12 @@ grub_hfsplus_read (grub_file_t file, char *buf, grub_size_t len)
 
 
 static grub_err_t
-grub_hfsplus_dir (grub_device_t device, const char *path,
-		  int (*hook) (const char *filename,
-			       const struct grub_dirhook_info *info))
+grub_hfsplus_dir (grub_device_t device, const char *path, 
+		  int (*hook) (const char *filename, int dir))
 {
   struct grub_hfsplus_data *data = 0;
   struct grub_fshelp_node *fdiro = 0;
-
+  
   auto int NESTED_FUNC_ATTR iterate (const char *filename,
 				     enum grub_fshelp_filetype filetype,
 				     grub_fshelp_node_t node);
@@ -913,18 +892,20 @@ grub_hfsplus_dir (grub_device_t device, const char *path,
 				enum grub_fshelp_filetype filetype,
 				grub_fshelp_node_t node)
     {
-      struct grub_dirhook_info info;
-      grub_memset (&info, 0, sizeof (info));
-      info.dir = ((filetype & GRUB_FSHELP_TYPE_MASK) == GRUB_FSHELP_DIR);
-      info.mtimeset = 1;
-      info.mtime = node->mtime;
-      info.case_insensitive = !! (filetype & GRUB_FSHELP_CASE_INSENSITIVE);
       grub_free (node);
-      return hook (filename, &info);
+      
+      if (filetype == GRUB_FSHELP_DIR)
+	return hook (filename, 1);
+      else 
+	return hook (filename, 0);
+      
+      return 0;
     }
 
+#ifndef GRUB_UTIL
   grub_dl_ref (my_mod);
-
+#endif
+  
   data = grub_hfsplus_mount (device->disk);
   if (!data)
     goto fail;
@@ -938,13 +919,15 @@ grub_hfsplus_dir (grub_device_t device, const char *path,
 
   /* Iterate over all entries in this directory.  */
   grub_hfsplus_iterate_dir (fdiro, iterate);
-
+  
  fail:
   if (data && fdiro != &data->dirroot)
     grub_free (fdiro);
   grub_free (data);
 
+#ifndef GRUB_UTIL
   grub_dl_unref (my_mod);
+#endif
 
   return grub_errno;
 }
@@ -957,59 +940,9 @@ grub_hfsplus_label (grub_device_t device __attribute__((unused))
   /* XXX: It's not documented how to read a label.  */
   return grub_error (GRUB_ERR_NOT_IMPLEMENTED_YET,
 		     "reading the label of a HFS+ "
-		     "partition is not implemented");
+		     "partiton is not implemented");
 }
 
-/* Get mtime.  */
-static grub_err_t
-grub_hfsplus_mtime (grub_device_t device, grub_int32_t *tm)
-{
-  struct grub_hfsplus_data *data;
-  grub_disk_t disk = device->disk;
-
-  grub_dl_ref (my_mod);
-
-  data = grub_hfsplus_mount (disk);
-  if (!data)
-    *tm = 0;
-  else
-    *tm = grub_be_to_cpu32 (data->volheader.utime) - 2082844800;
-
-  grub_dl_unref (my_mod);
-
-  grub_free (data);
-
-  return grub_errno;
-
-}
-
-static grub_err_t
-grub_hfsplus_uuid (grub_device_t device, char **uuid)
-{
-  struct grub_hfsplus_data *data;
-  grub_disk_t disk = device->disk;
-
-  grub_dl_ref (my_mod);
-
-  data = grub_hfsplus_mount (disk);
-  if (data)
-    {
-      *uuid = grub_malloc (16 + sizeof ('\0'));
-      grub_sprintf (*uuid, "%016llx",
-		    (unsigned long long)
-		    grub_be_to_cpu64 (data->volheader.num_serial));
-    }
-  else
-    *uuid = NULL;
-
-  grub_dl_unref (my_mod);
-
-  grub_free (data);
-
-  return grub_errno;
-}
-
-
 
 static struct grub_fs grub_hfsplus_fs =
   {
@@ -1019,18 +952,15 @@ static struct grub_fs grub_hfsplus_fs =
     .read = grub_hfsplus_read,
     .close = grub_hfsplus_close,
     .label = grub_hfsplus_label,
-    .mtime = grub_hfsplus_mtime,
-    .uuid = grub_hfsplus_uuid,
-#ifdef GRUB_UTIL
-    .reserved_first_sector = 1,
-#endif
     .next = 0
   };
 
 GRUB_MOD_INIT(hfsplus)
 {
   grub_fs_register (&grub_hfsplus_fs);
+#ifndef GRUB_UTIL
   my_mod = mod;
+#endif
 }
 
 GRUB_MOD_FINI(hfsplus)
