@@ -1,20 +1,21 @@
 /* ufs.c - Unix File System */
 /*
  *  GRUB  --  GRand Unified Bootloader
- *  Copyright (C) 2004,2005,2007  Free Software Foundation, Inc.
+ *  Copyright (C) 2004, 2005  Free Software Foundation, Inc.
  *
- *  GRUB is free software: you can redistribute it and/or modify
+ *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation, either version 3 of the License, or
+ *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
  *
- *  GRUB is distributed in the hope that it will be useful,
+ *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with GRUB.  If not, see <http://www.gnu.org/licenses/>.
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
 #include <grub/err.h>
@@ -38,8 +39,6 @@
 
 #define GRUB_UFS_ATTR_DIR	040000
 
-#define GRUB_UFS_VOLNAME_LEN	32
-
 /* Calculate in which group the inode can be found.  */
 #define inode_group(inode,sblock) ()
 
@@ -52,7 +51,7 @@
                            grub_le_to_cpu##bits2 (data->inode2.field))
 #define INODE_SIZE(data) INODE_ENDIAN (data,size,32,64)
 #define INODE_MODE(data) INODE_ENDIAN (data,mode,16,16)
-#define INODE_BLKSZ(data) (data->ufs_type == UFS1 ? 4 : 8)
+#define INODE_BLKSZ(data) (data->ufs_type == UFS1 ? 32 : 64)
 #define INODE_DIRBLOCKS(data,blk) INODE_ENDIAN \
                                    (data,blocks.dir_blocks[blk],32,64)
 #define INODE_INDIRBLOCKS(data,blk) INODE_ENDIAN \
@@ -88,12 +87,7 @@ struct grub_ufs_sblock
   /* The frags per cylinder group.  */
   grub_uint32_t frags_per_group;
   
-  grub_uint8_t unused7[488];
-
-  /* Volume name for UFS2.  */
-  grub_uint8_t volume_name[GRUB_UFS_VOLNAME_LEN];
-
-  grub_uint8_t unused8[660];
+  grub_uint8_t unused7[1180];
   
   /* Magic value to check if this is really a UFS filesystem.  */
   grub_uint32_t magic;
@@ -205,46 +199,40 @@ grub_ufs_get_file_block (struct grub_ufs_data *data, unsigned int blk)
 {
   struct grub_ufs_sblock *sblock = &data->sblock;
   unsigned int indirsz;
-  int log2_blksz; 
   
   /* Direct.  */
   if (blk < GRUB_UFS_DIRBLKS)
     return INODE_DIRBLOCKS (data, blk);
   
-  log2_blksz = grub_le_to_cpu32 (data->sblock.log2_blksz);
-  
   blk -= GRUB_UFS_DIRBLKS;
   
   indirsz = UFS_BLKSZ (sblock) / INODE_BLKSZ (data);
   /* Single indirect block.  */
   if (blk < indirsz)
     {
-      grub_uint32_t indir[UFS_BLKSZ (sblock) >> 2];
-      grub_disk_read (data->disk, INODE_INDIRBLOCKS (data, 0) << log2_blksz,
+      grub_uint32_t indir[UFS_BLKSZ (sblock)];
+      grub_disk_read (data->disk, INODE_INDIRBLOCKS (data, 0),
 		      0, sizeof (indir), (char *) indir);
-      return (data->ufs_type == UFS1) ? indir[blk] : indir[blk << 1];
+      return indir[blk];
     }
   blk -= indirsz;
   
   /* Double indirect block.  */
   if (blk < UFS_BLKSZ (sblock) / indirsz)
     {
-      grub_uint32_t indir[UFS_BLKSZ (sblock) >> 2];
+      grub_uint32_t indir[UFS_BLKSZ (sblock)];
       
-      grub_disk_read (data->disk, INODE_INDIRBLOCKS (data, 1) << log2_blksz,
+      grub_disk_read (data->disk, INODE_INDIRBLOCKS (data, 1),
 		      0, sizeof (indir), (char *) indir);
-      grub_disk_read (data->disk,
-      		      (data->ufs_type == UFS1) ?
-		      indir[blk / indirsz] : indir [(blk / indirsz) << 1],
+      grub_disk_read (data->disk,  indir[blk / indirsz],
 		      0, sizeof (indir), (char *) indir);
       
-      return (data->ufs_type == UFS1) ?
-	     indir[blk % indirsz] : indir[(blk % indirsz) << 1];
+      return indir[blk % indirsz];
     }
 
 
   grub_error (GRUB_ERR_NOT_IMPLEMENTED_YET,
-	      "ufs does not support triple indirect blocks");
+	      "ufs does not support tripple indirect blocks");
   return 0;
 }
 
@@ -253,9 +241,9 @@ grub_ufs_get_file_block (struct grub_ufs_data *data, unsigned int blk)
    POS.  Return the amount of read bytes in READ.  */
 static grub_ssize_t
 grub_ufs_read_file (struct grub_ufs_data *data,
-		    void NESTED_FUNC_ATTR (*read_hook) (grub_disk_addr_t sector,
+		    void (*read_hook) (unsigned long sector,
 				       unsigned offset, unsigned length),
-		    int pos, grub_size_t len, char *buf)
+		    int pos, unsigned int len, char *buf)
 {
   struct grub_ufs_sblock *sblock = &data->sblock;
   int i;
@@ -406,13 +394,13 @@ grub_ufs_lookup_symlink (struct grub_ufs_data *data, int ino)
 static grub_err_t
 grub_ufs_find_file (struct grub_ufs_data *data, const char *path)
 {
-  char fpath[grub_strlen (path) + 1];
+  char fpath[grub_strlen (path)];
   char *name = fpath;
   char *next;
   unsigned int pos = 0;
   int dirino;
   
-  grub_strcpy (fpath, path);
+  grub_strncpy (fpath, path, grub_strlen (path));
   
   /* Skip the first slash.  */
   if (name[0] == '/')
@@ -643,7 +631,7 @@ grub_ufs_open (struct grub_file *file, const char *name)
 
 
 static grub_ssize_t
-grub_ufs_read (grub_file_t file, char *buf, grub_size_t len)
+grub_ufs_read (grub_file_t file, char *buf, grub_ssize_t len)
 {
   struct grub_ufs_data *data = 
     (struct grub_ufs_data *) file->data;
@@ -662,30 +650,10 @@ grub_ufs_close (grub_file_t file)
 
 
 static grub_err_t
-grub_ufs_label (grub_device_t device, char **label)
+grub_ufs_label (grub_device_t device __attribute ((unused)),
+		char **label __attribute ((unused)))
 {
-  struct grub_ufs_data *data = 0;
-
-#ifndef GRUB_UTIL
-  grub_dl_ref (my_mod);
-#endif
-
-  *label = 0;
-
-  data = grub_ufs_mount (device->disk);
-  if (data)
-    {
-      if (data->ufs_type == UFS2)
-        *label = grub_strdup ((char *) data->sblock.volume_name);
-    }
-
-#ifndef GRUB_UTIL
-  grub_dl_unref (my_mod);
-#endif
-
-  grub_free (data);
-
-  return grub_errno;
+  return GRUB_ERR_NONE;
 }
 
 
