@@ -1,19 +1,20 @@
 /*
  *  GRUB  --  GRand Unified Bootloader
- *  Copyright (C) 1999,2000,2002,2005,2006,2007,2008,2010   Free Software Foundation, Inc.
+ *  Copyright (C) 1999,2000,2002,2005   Free Software Foundation, Inc.
  *
- *  GRUB is free software: you can redistribute it and/or modify
+ *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation, either version 3 of the License, or
+ *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
  *
- *  GRUB is distributed in the hope that it will be useful,
+ *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with GRUB.  If not, see <http://www.gnu.org/licenses/>.
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
 #ifndef GRUB_BOOT_MACHINE_HEADER
@@ -26,16 +27,28 @@
 #define GRUB_BOOT_MACHINE_BPB_START	0x3
 
 /* The offset of the end of BPB (BIOS Parameter Block).  */
-#define GRUB_BOOT_MACHINE_BPB_END	0x5a
+#define GRUB_BOOT_MACHINE_BPB_END	0x3e
 
-/* The offset of KERNEL_SECTOR.  */
-#define GRUB_BOOT_MACHINE_KERNEL_SECTOR	0x5c
+/* The offset of the major version.  */
+#define GRUB_BOOT_MACHINE_VER_MAJ	0x3e
 
 /* The offset of BOOT_DRIVE.  */
-#define GRUB_BOOT_MACHINE_BOOT_DRIVE	0x64
+#define GRUB_BOOT_MACHINE_BOOT_DRIVE	0x40
+
+/* The offset of FORCE_LBA.  */
+#define GRUB_BOOT_MACHINE_FORCE_LBA	0x41
+
+/* The offset of KERNEL_ADDRESS.  */
+#define GRUB_BOOT_MACHINE_KERNEL_ADDRESS	0x42
+
+/* The offset of KERNEL_SECTOR.  */
+#define GRUB_BOOT_MACHINE_KERNEL_SECTOR	0x44
+
+/* The offset of KERNEL_SEGMENT.  */
+#define GRUB_BOOT_MACHINE_KERNEL_SEGMENT	0x48
 
 /* The offset of BOOT_DRIVE_CHECK.  */
-#define GRUB_BOOT_MACHINE_DRIVE_CHECK	0x66
+#define GRUB_BOOT_MACHINE_DRIVE_CHECK	0x4b
 
 /* The offset of a magic number used by Windows NT.  */
 #define GRUB_BOOT_MACHINE_WINDOWS_NT_MAGIC	0x1b8
@@ -64,18 +77,6 @@
 #define GRUB_BOOT_MACHINE_KERNEL_ADDR	(GRUB_BOOT_MACHINE_KERNEL_SEG << 4)
 
 /* The size of a block list used in the kernel startup code.  */
-#define GRUB_BOOT_MACHINE_LIST_SIZE	12
-
-#ifndef ASM_FILE
-
-/* This is the blocklist used in the diskboot image.  */
-struct grub_boot_blocklist
-{
-  grub_uint64_t start;
-  grub_uint16_t len;
-  grub_uint16_t segment;
-} __attribute__ ((packed));
-
-#endif /* ! ASM_FILE */
+#define GRUB_BOOT_MACHINE_LIST_SIZE	8
 
 #endif /* ! BOOT_MACHINE_HEADER */
