@@ -1,7 +1,7 @@
 /* grub-probefs.c - probe a filesystem module for a given path */
 /*
  *  GRUB  --  GRand Unified Bootloader
- *  Copyright (C) 2005 Free Software Foundation, Inc.
+ *  Copyright (C) 2005,2006 Free Software Foundation, Inc.
  *
  *  GRUB is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -28,6 +28,7 @@
 #include <grub/pc_partition.h>
 #include <grub/machine/util/biosdisk.h>
 #include <grub/util/getroot.h>
+#include <grub/term.h>
 
 #include <stdio.h>
 #include <unistd.h>
@@ -52,6 +53,18 @@ grub_putchar (int c)
   putchar (c);
 }
 
+int
+grub_getkey (void)
+{
+  return -1;
+}
+
+grub_term_t
+grub_term_get_current (void)
+{
+  return 0;
+}
+
 void
 grub_refresh (void)
 {
